 -- ===============================================================

function love.load()

	-- game 
	game = {} 
	game.width = love.window.getWidth() 
	game.height = love.window.getHeight() 
	game.paddleWidth = 90 
	game.paddleHeight = 20 
	game.paddleSpeed = 10 
	game.brickWidth = 64
	game.brickHeight = 20
	game.ballsTotal = 5
	game.ballsCurrent = game.ballsTotal
	game.score = 0
	game.timer = nil
	game.timerLimit = nil
	game.timerLimitCycles = nil

	-- ball
	ball = {}
	ball.location = {0,0}
	ball.speed = nil
	ball.direction = 1
	ball.angle = 0
	ball.outOfBoundsFlag = false
	ball.size = 8;

	-- player
	player = {} 
	player.x = (game.width / 2) - (game.paddleWidth /2) 

  enemies = {}
	enemies.totalNum = 0

  for y= 1,3 do
      for x=0,15 do
	      enemy = {}
	      enemy.width = 40
	      enemy.height = 20
	      enemy.x = x * (enemy.width + 10) + 5
	      enemy.y = enemy.height + (y * 30)
	      if y == 1 then 
	      	enemy.r = 109
	      	enemy.g = 208
	      	enemy.b = 247
	      elseif y == 2 then 
	      	enemy.r = 244
	      	enemy.g = 154
	      	enemy.b = 194
	      else  
	      	enemy.r = 255
	      	enemy.g = 248
	      	enemy.b = 153
	      end	
	      table.insert(enemies, enemy)
	      enemies.totalNum = enemies.totalNum + 1
	    end  
  end

  -- set score and balls font
	font = love.graphics.newFont("gameFont.ttf", 25)
	love.graphics.setFont(font)
	-- set backgorund image
	love.graphics.setColor(255,255,255,255)
	backroundImage = love.graphics.newImage("backgroundImage.png")

	startGame()

end

function startGame() 
	print ('ball launched (speed and angle reset)')
	game.timer = 0
	game.timerLimit = 5
	game.timerLimitCycles = 0
	ball.speed = 5

	spawnBall()
end

function spawnBall()

	if game.ballsCurrent > 0 then
		ball.location = { math.random(0, love.window.getWidth()/2),
			math.random(180, love.window.getHeight()/2)
		}
		ball.direction = 1
		ball.angle = math.random(-3,3)
		if (ball.angle == 0) then ball.angle = 1 end
		print ('ball.angle = ' .. ball.angle .. ', ' .. game.ballsCurrent .. ' balls left')
		ball.outOfBoundsFlag = false
		game.ballsCurrent = game.ballsCurrent -1
	end	

end

 -- ===============================================================

function love.draw()

	-- draw background
	love.graphics.draw(backroundImage)

	-- score and lives
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(game.score, 20, 10)
	love.graphics.print(game.ballsTotal - game.ballsCurrent .. ' of ' .. game.ballsTotal, 710, 10)

	-- draw paddle
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle("fill", player.x, game.height - game.paddleHeight, game.paddleWidth, game.paddleHeight)

	-- draw ball
	love.graphics.circle("fill", ball.location[1], ball.location[2], ball.size, ball.size)	

	-- bricks
  for i,v in ipairs(enemies) do
  		love.graphics.setColor(v.r, v.g, v.b)
      love.graphics.rectangle("fill", v.x, v.y, v.width, v.height)
  end
		
end

 -- ===============================================================

function love.update(dt)

	local remEnemy = {}

	-- game.timerLimitCycles (to increase the ball speed by 1 every time a Cycle happens)
	game.timer = game.timer + dt
	if game.timer >= game.timerLimit then
		game.timerLimitCycles = game.timerLimitCycles + 1
		game.timer = 0
		ball.speed = ball.speed + 1
	  print ('increasing speed to ' .. game.timerLimitCycles)
	end

	-- keyboard input
	if love.keyboard.isDown("left") then
		player.x = player.x - game.paddleSpeed
		if player.x < 0 - (game.paddleWidth /2) then player.x = 0 - (game.paddleWidth /2) end
	elseif love.keyboard.isDown("right") then
		player.x = player.x + game.paddleSpeed
		if player.x > (love.window.getWidth() - (game.paddleWidth /2)) then 
			player.x = (love.window.getWidth() - (game.paddleWidth /2))
		end
	end

	-- ball X motion
	ball.location[1] = ball.location[1] + ball.angle

	if ball.location[1] < ball.size then
		ball.angle = ball.angle * -1	
	elseif ball.location[1] > love.window.getWidth() - ball.size then
		ball.angle = ball.angle * -1
	end

	-- ball Y motion
	if ball.direction == -1 then
		ball.location[2] = ball.location[2] - ball.speed;
	else
		ball.location[2] = ball.location[2] + ball.speed;	
	end

	-- border collision detection
	if ball.outOfBoundsFlag == false then
		if ball.location[2] > love.window.getHeight() - game.paddleHeight - ball.size then
			if ball.location[1] > player.x and ball.location[1] < player.x + game.paddleWidth then
				local paddleContactPoint = (player.x - ball.location[1]) * - 1
				print ( 'paddleContactPoint = ' .. paddleContactPoint)
				-- this hardcoced value needs to be computed based on paddle width
				if paddleContactPoint <= 20 then
					-- paddle hit on left end, so change veer more to left
					ball.direction = ball.direction * -1
					ball.angle = ball.angle - 2
				elseif paddleContactPoint >= 70 then
					-- paddle hit on right end, so change veer more to right					
					ball.direction = ball.direction * -1
					ball.angle = ball.angle + 2
				else
					-- paddle hit in middle so no angle change					
					ball.direction = ball.direction * -1
				end	
			else 
				ball.outOfBoundsFlag = true
			end	
		elseif ball.location[2] < ball.size then
			ball.direction = ball.direction * -1
		end
	else
			if ball.location[2] > love.window.getHeight() + 10 then
				startGame()	
			end	
	end	

	
	-- brick collision detection	
  for i,v in ipairs(enemies) do
      if CheckCollision (ball.location[1],ball.location[2],ball.size,ball.size,v.x,v.y,v.width,v.height) == true then

          -- mark that enemy for removal
          table.insert(remEnemy, i)
          ball.direction = ball.direction * -1
          print(ball.location[1] .. ' x ' .. ball.location[2])
          enemies.totalNum = enemies.totalNum - 1
					-- todo: make bricks different scores
          game.score = game.score + 10
      end
  end

  -- remove the marked enemies
  for i,v in ipairs(remEnemy) do
      table.remove(enemies, v)
      print ( 'bricks left = ' .. enemies.totalNum )
  end  

  -- are all bricks gone?
  if enemies.totalNum == 0 then
  	print ( 'you win!' )	
  end

  -- are all bricks gone?
  if game.ballsCurrent < 1 then
  	print ( 'no balls left' )	
  end  

end


-- https://love2d.org/wiki/BoundingBox.lua
function CheckCollision(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 < x2+w2 and
         x2 < x1+w1 and
         y1 < y2+h2 and
         y2 < y1+h1
end





